############################################################################
#    External method EventFolder/Extensions/rendercal.py
#    Ver 0.5  May 31 2000
#
#    License follows the Zope license, & use at your own risk
#    written and Copyright by Jeff Sasmor  (jeff@sasmor.com)
#
#    Two methods: renderMonth and renderYear.
#############################################################################

import DateTime

#############################################################################
#    R E N D E R M O N T H
#
#    Renders the month. It cycles thru all of the dates in the month.
#    Liberally borrows from ideas in CalendarTag.py from the
#    Calendar Tag product. It's just a lot simpler since it only
#    handles months and doesn't execute any code in the calling DTML
#    method (since it isn't a tag)
#
#    A R G S
#
#    passed-in date must have the month day = 1 --  e.g., Jan 1 2000.
#    date needs to be a DateTime object!!
#
#
#	srchmethod is the URL of a method (passed as a string) to call with
#   "?year/month/day" tacked on as the query string.
#   srchmethod should use the query string to talk to the Catalog and
#   get a list of events for the date.  This scheme is used to generate
#   the URLs for the calendar's links.
#
#    bodyFgColor and bodyBgColor are color of the cells
#    headingFgColor and headingBgColor are color of the heading
#
#    showYearName should = 'showyearname' to show the year names in the header or
#    anything else if not.  Usually you would use 'showyearname', the renderYear method
#    uses '' to avoid repeating the year for each month.
#
#    moderatedMode = 'on' to do ZCatalog searching with an ANDing term of reviewed:on
#
##################################################################################

def renderMonth(self, date, srchMethod, bodyFgColor, bodyBgColor, headingFgColor, headingBgColor, showYearName, moderatedMode ):

	date = date.earliestTime() + 0.5    #Adding 0.5 to earliestTime() turns out to be important: due to arithmetic precision issues (?)
										#if this isn't done then date = (somedate) time =00:00:00) sometimes date = date + 1 (below)
										#doesn't actually inc to the next day!!
	month = date.month()              	#get current month
	dayofweek = date.dow()	    		#init day-of-week of first date of the month (sunday==0 when dow() is used).
	daynames = ('S','M','T','W','T','F','S')   #tuple of day names
	
	#build the enclosing table and the MONTH [,YEAR] header
	s = '\n<table cellpadding="0" cellspacing="0" width="100%c">\n<tr><td valign="center" align="center" bgcolor="%s">\n<font color="%s">\n<b>&nbsp;' % ('%',headingBgColor,headingFgColor)
	if showYearName == 'showyear':
		s = s + '%s %d' % (date.Month(), date.year() )
	else:
		s = s + '%s' % date.Month()

	s = s + '&nbsp;</b>\n</font></td></tr>\n<tr><td align="center" valign="center" bgcolor="#000000">\n<font color="%s">' % bodyFgColor
	
	
	#build the beginning of the calendar
	s = s + '<table border="0"  width="100%" cellspacing="2" cellpadding="4"><tr>\n' #open the table and the first row
		                                                                                      		
	for d in daynames:
		s = s + '<th bgcolor="%s">%s</th>\n' % (bodyBgColor,d)              #make the row S, M T, W, T, F S

	s = s + '</tr>'						#close the row
	
	#now build the rest of the table
	
	if dayofweek > 0:           		#if the first day is not sunday we need to use up some table day-columns until we reach the correct day
	    #fill in the unused days of the month with blanks. It's O to have colspan=1 but NOT = 0 (browser-specific probs!)
	    s = s + '<tr><td colspan="%s" bgcolor="%s">&nbsp;</td>\n' % (dayofweek,bodyBgColor)
	else:
	    s = s + '<tr>\n'					    #this must be Sunday...
	

	#loop thru all the days till the end of the month and put in a link to event-display method (if there are event(s) for that day)
	#or just the day # (if not).

	#invariant part of arg list to Catalog.searchResults()
	arglist = {'event_date_usage':'range:min:max', 'meta_type':'EvDocDate'}
		
	while date.month() == month:		#while we're in the current month
		if dayofweek == 7:				#if we have reached the end of the week
			s = s + '</tr><tr>\n'		#end the current table row and start a new one
			dayofweek = 0				#reinit the current day to zero
		
		s = s + '<td bgcolor="%s">' % (bodyBgColor)		#start a new form column-cell for this day
		
		#set 'event_date' to range for this day
		arglist['event_date']=[date.earliestTime(), date.latestTime()]

		results = self.Catalog.searchResults(arglist) #and do the search
		if results:     #ok, there's a date match, BUT...
		#If moderatedMode=='on' we have to chk the parent obj (the event doc) to see if it has the reviewed property == on
			if moderatedMode=='on':			
				found = 'no'
				for rid in results:
					obj = self.Catalog.getobject(rid.data_record_id_)
					pn = obj.getParentNode()
					if pn.reviewed == 'on':
						found = 'yes'
						#make up a URL for this date
						s = s +  '<a href="%s?%s/%s/%s">%s</a></td>\n' % (srchMethod, date.year(),date.mm(),date.dd(),date.dd() )
						break;		#drops down to (X marks the spot) below
				if found == 'no':	#no reviewed event was found, just print the day-number
					s = s + '%s</td>\n' % date.dd()
			else:	#always print a link if we found a date and moderatedMode != on
				s = s +  '<a href="%s?%s/%s/%s">%s</a></td>\n' % (srchMethod, date.year(),date.mm(),date.dd(),date.dd() )
					
		else: #or just print the day-number if there's no event for this date
			s = s + '%s</td>\n' % date.dd()
	    #X marks the spot
		dayofweek = dayofweek + 1   	#increment the day-of-week
		date =  date + 1				#increment the date by 1			

    #we're done with the month.  Do we need to add blank columns to the month because the last day of the month was not Saturday?
	if dayofweek < 7:
		s = s + '<td colspan="%s" bgcolor="%s">&nbsp;</td>' % (7 - dayofweek, bodyBgColor)

	s = s + '</tr></table></font></td></tr></table>'	#terminate the final rows & inner table, the curently-open font tag, and the outer table

	return s				            #return the formatted calendar month to the caller
	
	
	
	
	
#############################################################################
#    R E N D E R Y E A R

#    Renders the year. Basically just a wrapper around renderMonth.
#
#    A R G S
#
#    Same as renderMonth, except that the 'date' parameter is replace with 'year' where 'year'
#    is a string (NOT A DATETIME OBJECT) with just the year (using all four digits; e.g., '2000'.
##################################################################################

def renderYear(self, year, srchMethod, bodyFgColor, bodyBgColor, headingFgColor, headingBgColor, breaks, moderatedMode):

	s = '\n<table width="100%c" ><tr><td valign="top">\n ' % ('%')
	
	test = int(breaks)   				#lose that pesky 'number coercion' error
	if test<1 or test>12:				#sanity check for row breaks
		test = 3;
	for i in range(1,13):				#iterate over the months
		tdate = '%s/%d/1' % (year,i)	#make up a date string = year/month/1
		date =  DateTime.DateTime(tdate) #and make a DateTime obj out of it
		#render a month
		s = s + self.renderMonth(date, srchMethod,bodyFgColor, bodyBgColor, headingFgColor, headingBgColor, '', moderatedMode )
		if i % test == 0:				#if test passes then we want to end a row of months and begin another
			s = s + '</td></tr><tr><td valign="top">\n'
		else:
			s = s + '</td><td valign="top">\n'

	s = s + '</td></tr></table>\n'		#clean up...
	return s		
		
