#utils.py
############################################################################
#    External method EventFolder/Extensions/utils.py
#    Ver 0.5  June 8 2000
#
#    License follows the Zope license, & use at your own risk
#    written and Copyright by Jeff Sasmor  (jeff@sasmor.com)
#
#    lastDayOfMonth: pass a DateTime object and it returns 
#                    '28','29','30', or'31', depending on the month
#
#    dateRangeArgs: arg list builder for the neatManageView's catalog search (handles date ranges).
#
#	 wrapSearchResults: special tricky wrapper for Catalog.searchResults for
#                         neatManageView's catalog search. Also handles selection of
#                         subject and reviewed state.
#
#    makeItAList: wrapper around builtin fcn 'list' (converts sequence into a
#                 list) can't call this in DTML (maybe this is just a lack of knowledge?).
#
#    deleteEvents: wrapper around manage_delObjects (in OFS/ObjectManager.py)
#
#    subobjTypes: creates a string with # of subobjects and types (review or update)
#
#    makeBatcherString: creates a string with batching variables, used in neatManageView
#
#############################################################################

import DateTime
import string
import OFS.ObjectManager
import App.Management
from Globals import MessageDialog


def lastDayOfMonth(self,date):
	
	m = date.month()
	leap = date.isLeapYear()
	if m == 9 or m == 4 or m == 6 or m == 11:
		return '30'
	if m == 2:
		if leap:
			return '29'
		else:
			return '28'
	else:
		return '31'
		

def dateRange(self, date1, date2,  bsize,REQUEST):

	msg = ''
	dt1 = None
	dt2 = None
	status='ok'
	baseDict = {'meta_type':'EvDocDate','batch_size':bsize}
	
	
	if date1:
		try: dt1=DateTime.DateTime(string.strip(date1))
		except:
			msg= 'Start: \"%s\" had incorrect date format: try YYYY/MM/DD  HH:MM [AM/PM]' % date1
			status='nok'	

	if dt1 and date2:  #means that date1 was valid and there's a date2 string
		try: dt2=DateTime.DateTime(string.strip(date2))
		except:
			msg= 'End: \"%s\" had incorrect date format: try YYYY/MM/DD  HH:MM [AM/PM]' % date2
			status='nok'
	
	if dt1 and not dt2 and (status=='ok'):				#only date1 was provided
		dt1 = dt1.earliestTime()	#set dt objs to earliest and latest times on same day
		dt2 = dt1.latestTime()

	if dt1 and dt2:					#both date1 & date 2 were provided
		dt1 = dt1.earliestTime()	#set dt objs to earliest and latest times on their respective dates
		dt2 = dt2.latestTime()
		
	if date2 and not dt1 and (status=='ok'):				#this is an error
		msg = 'Use Start date only or both. End date by itself is not acceptable'
		status='nok'
	
	if status=='ok':
		if dt1 and dt2:
			baseDict['event_date']=[dt1,dt2]
			baseDict['event_date_usage']='range:min:max'
		return baseDict;
	else:
		return {'Error':msg}

		
		
def wrapSearchResults(self,arglist,subjectFilter,reviewFilter):

	results = self.Catalog.searchResults(arglist)	#get all EvDocDate instances for the date range
	
	#now we have to
	#   A. get the parent EventDoc for each EvDocDate
	#   B. discard the EvDocDate if we already have the parent (num EvDocDates per EventDoc can be >1)
	#   C. Optionally see if the EventDoc is reviewed
	#   D. put the EventDoc in the list

	if not results:		#this could happen...
		return []
		
	#implied else:
	evList = []			#init the list of EventDocs to an empty list
	for item in results:	#loop thru the results
		obj = self.Catalog.getobject(item.data_record_id_)	#get the object from the result item (an EvDocDate)
		pn = obj.getParentNode()							#and get the object's parent (an EventDoc)
		flag = '0'  		
		
		#you'd think that you could replace the following with     if pn in edlist    but it doesn't work. Mystery.
		#nor does a loop that has the comparison as Node == pn.  These should be the same instance, so why?
		for event in evList:					#if this EventDoc isn't already in the list
			if event.id == pn.id:
				flag = '1'
				break
		if flag == '0':
			evList.append(pn)				#else always append it.
			
	#now we filter for subject and review
	if subjectFilter == 'IGNORE' and reviewFilter == 'all':	#no filtering
		return evList;

	#implied else:	
	evListOut = []							#output list: init to empty
	for event in evList:
		if (subjectFilter == 'IGNORE') or (subjectFilter in event.subject): #we're not filtering on subject (or we are) and there's a match
			if (reviewFilter == 'all') or (reviewFilter == event.reviewed):		#if no review status filtering or (if there is) and desired state is matchec.
				evListOut.append(event);	#copy to the output list
	
	return evListOut	
	
   	

	
		
		
				
	
def makeItAList(self,input):
	return list(input)


def deleteEvents(self,REQUEST):
	if not REQUEST.has_key('ids'):
		REQUEST['RESPONSE'].redirect(REQUEST['URL1']+'/neatManageView?managementStatusMessage=E:No items were selected for deletion')		
	else:	
		self.manage_delObjects(REQUEST['ids'],None)
		REQUEST['RESPONSE'].redirect(REQUEST['URL1']+'/neatManageView')
		

def subobjTypes(self,parentObj):

	updates = 0;
	reviews = 0;
	subobjVals = parentObj.objectValues('EvDocAdder') 	#get the subobjects of the parent
	for t in subobjVals:
		if  t.isUpdate == 'on':
			updates = updates + 1
		else:
			reviews = reviews + 1
	if updates == 0:
		return '%d review%s' % (reviews, (((reviews>1)and's') or '') )
	elif reviews == 0:
		return '%d update%s' % (updates, (((updates>1)and's') or '') )
	else:		
		return '%d review%s<br>%d update%s' % (reviews,(((reviews>1)and's') or ''),updates,(((updates>1)and's') or '') )




def makeBatcherString(self,qs1,name1,qs2,name2):
	
	s = ''		#default to empty return string
	
	if qs1:
		s = s + '&%s=%s' % (name1,qs1)
	if qs2:	
		s = s + '&%s=%s' % (name2,qs2)
	return s



	
