#validateform.py
############################################################################
#    External method EventFolder/Extensions/validateform.py
#    Ver 0.5  June 6 2000
#
#    License follows the Zope license, & use at your own risk
#    written and Copyright by Jeff Sasmor  (jeff@sasmor.com)
#
#    One method: validateDocumentForm.
#############################################################################

import DateTime
import string



#Simple check for at least a reasonably-formatted phone number. Returns error string or ''	
def phoneNumCheck(self,s,name):
	
	slen= len(s)
	msg = ''
	if slen > 0:
		hyphens = string.count(s,'-')		#how many hyphens in the phone number
		if hyphens == 0 and slen < 7:
			msg = 'phone # must be at least 7 characters'
		elif hyphens == 1 and slen < 8:
			msg =  'phone # must be at least 8 characters if there is one hyphen in it'
		elif hyphens == 2 and slen < 12:
			msg = 'phone # must be at least 12 characters if there are two hyphens in it'
	
		if msg != '':
			return '%s %s' % (name,msg)
		else:
			return ''
	else:
		return ''
	
	
#Dumb check for some level of validity to email address- could be better....
def emailAddrCheck(self,s,name):

	slen = len(s)
	if slen > 0:
		ats = string.count(s,'@')
		if ats != 1:
			return '%s email address needs at least one @!' % (name)
		else:
			return ''
	else:
		return ''




#############################################################################
#    validateDocumentForm
#
#    validates the form data submitted when an EventDocument or EventDocumentAddendum
#	 is created or edited.
#
#	 Returns an empty list if no error, or a list of error strings
#    (may be just one...) describing the errors found.
#
#    A R G S
#
#    REQUEST  the request data object.
#
#    maxTitleChars = the max number of chars for the event title: only applies to the
#                    new event form
#
#    formType = 'EVENTFORM' or 'EVENTEDITFORM' or 'ADDERFORM'
#
#    most of these tests should be obvious (?) so there aren't a lot of comments.
##################################################################################

def validateDocumentForm(self, formType, maxTitleChars, REQUEST):
	
	L = []		#initialize return value to an empty list
	
	#First we check the title for existence and not too short.	
	if REQUEST.has_key('evtitle'):
		t = string.strip(REQUEST['evtitle'])
		lt = len(t)
		
		if lt < 4:
			L.append('Event title missing or too short: must be at least 4 characters')
		elif lt > maxTitleChars:
			s = 'Title too long: must be less than %d characters' % maxTitleChars
			L.append(s)		
		
   #Next chk the date for existence and then for correctness using try/except clause
   #to avoid funky-looking Zope error pages.

	if REQUEST.has_key('evdates'):
		index = 0
		list = REQUEST['evdates']
		numdates = 0	
		
		for date in list:
			sd = string.strip(date)
			if sd != '':
				try: junk=DateTime.DateTime(sd)
				except: L.append('Incorrect date format for [%s] on line %d of the <b>Date</b> field: try YYYY/MM/DD  HH:MM [AM/PM]' % (sd,index+1))
				else: numdates = numdates + 1
			index = index + 1
			
		if numdates == 0:
			L.append('There were no valid dates in the Date & Time field')
		   	if list[0] == '':  #can't allow first line to be blank: this date is used to make up the ID for an event.
				L.append('Also, first line of <b>Date & Time</b> field cannot be blank')

		elif list[0] == '':  #can't allow first line to be blank: this date is used to make up the ID for an event.
			L.append('First line of <b>Date & Time</b> field cannot be blank')
			
			

 	
	#Next we check to see if at least one subject field has been selected
	if formType == 'EVENTFORM' or formType == 'EVENTEDITFORM' :		#allows diffentiation between the different forms
		if 	not REQUEST.has_key('subject'):
			L.append('You need to specify at least one subject field')

	#Now check to see author's name AND (email OR phone) exist.

	if REQUEST.has_key('creator_name'):
		cn = string.strip(REQUEST['creator_name'])
		if len(cn) == 0:
			L.append('Author\'s name is missing')

	if REQUEST.has_key('creator_email'):
		ce = string.strip(REQUEST['creator_email'])
	else:
		ce = ''
		
	if REQUEST.has_key('creator_phone'):
		cp = string.strip(REQUEST['creator_phone'])
	else:
		cp = ''
	
	if len(ce) == 0 and len(cp) == 0:
		L.append('Either Author email or phone must be specified')		
	else:
		#check phone number
		s = phoneNumCheck(self,cp,'Author\'s')
		if s != '':
			L.append(s)

		#check email address
		s = emailAddrCheck(self,ce,'Author\'s')
		if s != '':
			L.append(s)
						
	# if the contact=author check box is on we can skip the next bunch of tests to validate contact info
	
	if formType == 'EVENTFORM' or formType == 'EVENTEDITFORM':		#allows diffentiation between the different forms
		if not REQUEST.has_key('contact_same_as_creator'):	#if checkbox is OFF we do:
			#a check to see if contact's name AND (email OR phone) exist.

			if REQUEST.has_key('contact_name'):
				cn = string.strip(REQUEST['contact_name'])
				if len(cn) == 0:
					L.append('Contact\'s name is missing')

			if REQUEST.has_key('contact_email'):
				ce = string.strip(REQUEST['contact_email'])
			else:
				ce = ''
		
			if REQUEST.has_key('contact_phone'):
				cp = string.strip(REQUEST['contact_phone'])
			else:
				cp = ''
	
			if len(ce) == 0 and len(cp) == 0:
				L.append('Either Contact email or phone must be specified')		
			else:
				#check phone number
				s = phoneNumCheck(self,cp,'Contact\'s')
				if s != '':
					L.append(s)

				#check email address
				s = emailAddrCheck(self,ce,'Contact\'s')
				if s != '':
					L.append(s)	

	#finally we do a simple check to see that there's something in the event description or comment_body
	if REQUEST.has_key('text_content'):
		tc = string.strip(REQUEST['text_content'])
		ltc = len(tc)
		if ltc < 8:
			if REQUEST.has_key('isAddendum'):
				L.append('The message was missing or really short: say a little more')
			else:
				L.append('Event description was missing or really short: say a little more')

			

	return L

	


